.form-window {
  width: 350px;
  margin: 100px auto;
}

.input-group-addon {
  i {
    &.fa {
      width: 12px;
    }
  }
}

.errorlist {
  margin: 5px 0 15px 0;
  padding: 8px;
  background-color: #f2dede;
  border-color: #eed3d7;
  color: #b94a48;
  list-style: none;
  .border-radius(4px);

  li {
    margin-bottom: 5px;

    &:last-child {
      margin-bottom: 0;
    }
  }
}