{% extends 'dartcms/views/insert.html' %}

{% load static %}

{% block js %}
    {{ block.super }}
    <script type="text/javascript" src="{% static 'dartcms/js/apps/pages/form.js' %}"></script>
{% endblock %}

{% block page_content %}
    {{ block.super }}
    <div id="id-context-variables" data-loading-url="{% url 'dartcms:pages:load_module_params' %}"></div>
{% endblock %}